@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #pragma once
 
 #include <c10/core/Allocator.h>
@@ -11,32 +12,6 @@
 
 namespace torch::cuda::CUDAPluggableAllocator {
 
-using MallocFuncType = void*(size_t, int, hipStream_t);
-using FreeFuncType = void(void*, size_t, int, hipStream_t);
-
-// A CUDAPluggableAllocatorDeleterContext object is used as the `ctx`
-// argument for DataPtr. We need context because a user can use
-// multiple allocators in the same PyTorch program, and
-// the allocators can have different free functions, such as:
-// free, hipFree, cudaFreeAsync, ncclMemFree etc.
-struct TORCH_HIP_CPP_API CUDAPluggableAllocatorDeleterContext {
-  explicit CUDAPluggableAllocatorDeleterContext(
-      std::function<FreeFuncType> free_fn,
-      void* data,
-      size_t size,
-      int device,
-      hipStream_t stream);
-
-  void free();
-
- private:
-  std::function<FreeFuncType> free_fn_;
-  void* data_;
-  size_t size_;
-  int device_;
-  hipStream_t stream_{};
-};
-
 #if defined(USE_ROCM)
 using streamType = c10::hip::HIPStream;
 #else
@@ -49,8 +24,8 @@ getCurrentAllocator();
 TORCH_HIP_CPP_API std::shared_ptr<
     c10::hip::HIPCachingAllocator::HIPAllocator>
 createCustomAllocator(
-    std::function<MallocFuncType> alloc_fn,
-    std::function<FreeFuncType> free_fn);
+    std::function<void*(size_t, int, hipStream_t)> alloc_fn,
+    std::function<void(void*, size_t, int, hipStream_t)> free_fn);
 TORCH_HIP_CPP_API void changeCurrentAllocator(
     const std::shared_ptr<c10::hip::HIPCachingAllocator::HIPAllocator>&
         allocator);
@@ -69,8 +44,8 @@ struct _AllocationMetadata {
 struct TORCH_HIP_CPP_API CUDAPluggableAllocator
     : public c10::hip::HIPCachingAllocator::HIPAllocator {
   CUDAPluggableAllocator(
-      std::function<MallocFuncType> alloc_fn,
-      std::function<FreeFuncType> free_fn);
+      std::function<void*(size_t, int, hipStream_t)> alloc_fn,
+      std::function<void(void*, size_t, int, hipStream_t)> free_fn);
 
   CUDAPluggableAllocator(CUDAPluggableAllocator& other);
   CUDAPluggableAllocator(CUDAPluggableAllocator&& other) = delete;
@@ -114,6 +89,8 @@ struct TORCH_HIP_CPP_API CUDAPluggableAllocator
   bool initialized() override;
   double getMemoryFraction(c10::DeviceIndex device) override;
   void setMemoryFraction(double fraction, c10::DeviceIndex device) override;
+  std::vector<c10::hip::HIPCachingAllocator::StreamSegmentSize>
+  getExpandableSegmentSizes(c10::DeviceIndex device) override;
   void emptyCache(c10::hip::MempoolId_t mempool_id = {0, 0}) override;
   void enable(bool) override {}
   bool isEnabled() const override {
@@ -173,8 +150,8 @@ struct TORCH_HIP_CPP_API CUDAPluggableAllocator
   void copy_data(void* dest, const void* src, std::size_t count) const final;
 
  protected:
-  std::function<MallocFuncType> alloc_fn_;
-  std::function<FreeFuncType> free_fn_;
+  std::function<void*(size_t, int, hipStream_t)> alloc_fn_;
+  std::function<void(void*, size_t, int, hipStream_t)> free_fn_;
   std::function<void(int)> init_fn_;
   std::function<void()> reset_fn_;
   std::function<void(double, int)> memory_fraction_fn_;
@@ -192,3 +169,7 @@ struct TORCH_HIP_CPP_API CUDAPluggableAllocator
   bool initialized_ = false;
 };
 } // namespace torch::cuda::CUDAPluggableAllocator
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
