@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #pragma once
 
 #include <memory>
@@ -26,3 +27,7 @@ std::shared_ptr<Source> findSourceInArchiveFromQualifier(
     const std::string& qualifier);
 
 } // namespace torch::jit
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
