@@ -23,7 +23,8 @@ import operator
 import textwrap
 import traceback
 import types
-import unittest
+from collections.abc import Sequence
+from contextlib import nullcontext
 from typing import TYPE_CHECKING
 
 import sympy
@@ -45,7 +46,7 @@ from torch.utils._python_dispatch import is_traceable_wrapper_subclass
 from .. import config, graph_break_hints, variables
 from .._trace_wrapped_higher_order_op import trace_wrapped
 from ..exc import (
-    unimplemented_v2,
+    unimplemented,
     UnknownPropertiesDuringBackwardTrace,
     UserError,
     UserErrorType,
@@ -62,12 +63,13 @@ from ..utils import (
     object_has_getattribute,
     product,
     proxy_args_kwargs,
+    raise_args_mismatch,
     set_example_value,
     tensortype_to_dtype,
 )
-from .base import AttributeMutationNew, VariableTracker
+from .base import AttributeMutationNew, ValueMutationNew, VariableTracker
 from .constant import ConstantVariable
-from .lists import SizeVariable
+from .lists import ListIteratorVariable, SizeVariable
 from .user_defined import UserDefinedClassVariable
 
 
@@ -81,6 +83,8 @@ if TYPE_CHECKING:
     from torch._dynamo.codegen import PyCodegen
     from torch._dynamo.symbolic_convert import InstructionTranslator
 
+    from .functions import UserFunctionVariable
+
 
 log = logging.getLogger(__name__)
 
@@ -202,6 +206,19 @@ class TensorVariable(VariableTracker):
             _is_name_set = self.proxy.node.op == "placeholder"
         self._is_name_set: bool = _is_name_set
 
+    def synchronize_attributes(self, tx, target_cls=None):
+        from .builder import get_specialized_props, infer_subclass_type
+
+        if target_cls is None:
+            target_cls = type(self)
+
+        example_value = self.proxy.node.meta.get("example_value")
+        specialized_props = get_specialized_props(
+            target_cls, tx, example_value, infer_subclass_type(example_value)
+        )
+        for k, v in specialized_props.items():
+            setattr(self, k, v)
+
     def debug_repr(self):
         # TODO: strip off fake tensor from repr here
         return repr(self.proxy.node.meta["example_value"])
@@ -212,6 +229,9 @@ class TensorVariable(VariableTracker):
     def python_type(self):
         return self.class_type
 
+    def is_tensor(self) -> bool:
+        return True
+
     @staticmethod
     def specialize(value: torch.Tensor):
         props = {
@@ -234,7 +254,7 @@ class TensorVariable(VariableTracker):
 
         if is_sparse_any(value) and not has_free_symbols(value):
             props["_size"] = tuple(
-                [int(s) if is_symbolic(s) else s for s in value.size()]
+                int(s) if is_symbolic(s) else s for s in value.size()
             )
         elif not has_free_symbols(value):
             # this is a fully static shape, and the keys on props here inform specialization.
@@ -246,7 +266,8 @@ class TensorVariable(VariableTracker):
             props["_size"] = tuple(
                 # the non is_symbolic case applies to the jagged layout
                 # NestedTensor case as singleton ints are not symbolic
-                [int(s) if is_symbolic(s) else s for s in value.size()]
+                int(s) if is_symbolic(s) else s
+                for s in value.size()
             )
             props["stride"] = tuple(value.stride())
             if torch._C._functorch.is_batchedtensor(value):
@@ -255,11 +276,9 @@ class TensorVariable(VariableTracker):
                 props["is_contiguous"] = None
             else:
                 props["is_contiguous"] = tuple(
-                    [
-                        x
-                        for x in torch._prims_common._memory_formats
-                        if value.is_contiguous(memory_format=x)
-                    ]
+                    x
+                    for x in torch._prims_common._memory_formats
+                    if value.is_contiguous(memory_format=x)
                 )
         return props
 
@@ -300,7 +319,7 @@ class TensorVariable(VariableTracker):
             # eval("super(L['mod'].model.model.encoder.embed_positions.forward__class__,
             # L['mod'].model.model.encoder.embed_positions)", scope)
             # Which is incorrect, and violates the invariant that all sources should be eval()-able against the scope.
-            _input_associated_real_value = eval(self.source.name(), scope)
+            _input_associated_real_value = eval(self.source.name, scope)
         except Exception as exc:
             raise NotImplementedError from exc
 
@@ -376,7 +395,7 @@ class TensorVariable(VariableTracker):
             return ConstantVariable.create(self.is_nested)
 
     def method_attr_retain_grad(self, tx):
-        unimplemented_v2(
+        unimplemented(
             gb_type="Tensor.retain_grad() with AOTDispatcher",
             context=f"var_getattr {self} retain_grad",
             explanation="`Tensor.retain_grad()` does not work with AOTDispatcher.",
@@ -390,7 +409,7 @@ class TensorVariable(VariableTracker):
 
     def method_attr_grad_fn(self, tx):
         if self.has_grad_fn:
-            unimplemented_v2(
+            unimplemented(
                 gb_type="Tensor with grad_fn()",
                 context=f"var_getattr {self} grad_fn",
                 explanation="Dynamo does not support tracing tensors with a grad_fn directly.",
@@ -414,7 +433,7 @@ class TensorVariable(VariableTracker):
         # Today, var_getattr returns GetAttrVariable for both non-existent
         # attributes and existing attributes. This is a bug and requires more
         # deep dive.
-        if name in ("size", "stride"):
+        if name in all_tensor_attrs:
             return ConstantVariable(True)
 
         try:
@@ -437,7 +456,7 @@ class TensorVariable(VariableTracker):
     def var_getattr(self, tx: "InstructionTranslator", name):
         if self.is_strict_mode(tx):
             if name in self._strict_mode_banned_ops():
-                unimplemented_v2(
+                unimplemented(
                     gb_type="Strict mode banned op",
                     context=f"var_getattr {self} {name}",
                     explanation=f"Getattr invocation '{name}' in strict mode is not supported.",
@@ -505,7 +524,7 @@ class TensorVariable(VariableTracker):
                 # these attributes are implemented under tp_getset, which appear
                 # as `getset_descriptor`s, (compared to, say, methods which appear
                 # as `method_descriptor`s)
-                if type(static_attr) != types.GetSetDescriptorType:
+                if type(static_attr) is not types.GetSetDescriptorType:
                     return None
 
                 proxy = GetAttrVariable.create_getattr_proxy(self.as_proxy(), name)
@@ -527,7 +546,7 @@ class TensorVariable(VariableTracker):
 
     def call_id(self, tx):
         if not self.source:
-            unimplemented_v2(
+            unimplemented(
                 gb_type="Unsupported call_id() without source",
                 context=f"call_id {self}",
                 explanation="call_id() not supported for sourceless TensorVariable.",
@@ -537,9 +556,9 @@ class TensorVariable(VariableTracker):
         # For local source, we associate the real value. We use this real value
         scope = {"L": tx.output.local_scope, "G": tx.output.global_scope}
         try:
-            _input_associated_real_value = eval(self.source.name(), scope)
+            _input_associated_real_value = eval(self.source.name, scope)
         except Exception as exc:
-            unimplemented_v2(
+            unimplemented(
                 gb_type="Error getting associated real value",
                 context=f"call_id {self}",
                 explanation="Dynamo encountered an error while trying to "
@@ -549,7 +568,7 @@ class TensorVariable(VariableTracker):
             )
 
         if _input_associated_real_value is None:
-            unimplemented_v2(
+            unimplemented(
                 gb_type="call_id() without associated real value",
                 context=f"call_id {self}",
                 explanation="Dynamo could not find an associated real value for the tensor.",
@@ -581,11 +600,14 @@ class TensorVariable(VariableTracker):
             dyn_length = self.call_method(tx, "size", [ConstantVariable.create(0)], {})
             # SymNodeVariable for symbolic sizes, ConstantVariable for constants OR values produced through
             # symbolic_shapes, but that end up as int/sympy.Integer
-            assert isinstance(dyn_length, (SymNodeVariable, ConstantVariable))
+            assert (
+                isinstance(dyn_length, SymNodeVariable)
+                or dyn_length.is_python_constant()
+            )
             if isinstance(dyn_length, SymNodeVariable):
                 length = dyn_length.evaluate_expr(tx.output)
             else:
-                length = dyn_length.value
+                length = dyn_length.as_python_constant()
 
         if idxes is None:
             idxes = range(length)
@@ -598,6 +620,16 @@ class TensorVariable(VariableTracker):
             for i in idxes
         ]
 
+    def call_tree_map(
+        self,
+        tx,
+        tree_map_fn: "UserFunctionVariable",
+        map_fn,
+        rest,
+        tree_map_kwargs,
+    ) -> "VariableTracker":
+        return map_fn.call_function(tx, [self, *rest], {})
+
     def valid_size(self):
         return self._size is not None
 
@@ -618,14 +650,14 @@ class TensorVariable(VariableTracker):
         self,
         tx,
         name,
-        args: "list[VariableTracker]",
+        args: Sequence[VariableTracker],
         kwargs: "dict[str, VariableTracker]",
     ) -> "VariableTracker":
         from .builder import SourcelessBuilder, VariableBuilder
         from .torch_function import can_dispatch_torch_function, dispatch_torch_function
 
         if self.is_strict_mode(tx) and name in self._strict_mode_banned_ops():
-            unimplemented_v2(
+            unimplemented(
                 gb_type="Illegal method invocation in strict mode",
                 context=f"call_method {self} {name} {args} {kwargs}",
                 explanation="Dynamo currently does not support this method "
@@ -669,7 +701,7 @@ class TensorVariable(VariableTracker):
         # discussions in #151432 for more details.
         # We graph break for now since this use case is uncommon.
         if name == "random_":
-            unimplemented_v2(
+            unimplemented(
                 gb_type="Tensor.random_ op",
                 context=f"Tensor.{name}({args=}, {kwargs=})",
                 explanation="This is currently not supported.",
@@ -679,7 +711,7 @@ class TensorVariable(VariableTracker):
                 ],
             )
         elif name == "uniform_" and "from" in kwargs:
-            unimplemented_v2(
+            unimplemented(
                 gb_type="Tensor.uniform_ op called with `from` keyword",
                 context=f"Tensor.{name}({args=}, {kwargs=})",
                 explanation="This is currently not supported.",
@@ -699,7 +731,7 @@ class TensorVariable(VariableTracker):
                 if result:
                     return result
             except TypeError as e:
-                unimplemented_v2(
+                unimplemented(
                     gb_type="Unhandled args for method",
                     context=f"call_method {self} {name} {args} {kwargs}",
                     explanation="Dynamo encountered an error while calling "
@@ -790,7 +822,7 @@ class TensorVariable(VariableTracker):
 
     def method_is_inference(self):
         if config.fake_tensor_disable_inference_mode:
-            unimplemented_v2(
+            unimplemented(
                 gb_type="Encountered tensor.is_inference() during tracing",
                 context="",
                 explanation="tensor.is_inference() is not supported",
@@ -876,7 +908,7 @@ class TensorVariable(VariableTracker):
                 object(), var, mutation_type_cls=AttributeMutationNew
             )
             return var
-        unimplemented_v2(
+        unimplemented(
             gb_type="Argument of `as_subclass` must be a non-dispatcher-style tensor subclass",
             context=f"{self}.as_subclass({cls})",
             explanation="Currently not supported",
@@ -896,7 +928,7 @@ class TensorVariable(VariableTracker):
 
     def method_numpy(self, *, force=False):
         if not config.trace_numpy:
-            unimplemented_v2(
+            unimplemented(
                 gb_type="Tensor.numpy() with trace_numpy=False",
                 context=f"call_method {self} numpy",
                 explanation="`Tensor.numpy()` was called, but the `trace_numpy` "
@@ -907,7 +939,7 @@ class TensorVariable(VariableTracker):
                 ],
             )
         if not np:
-            unimplemented_v2(
+            unimplemented(
                 gb_type="Tensor.numpy() without NumPy installed",
                 context=f"call_method {self} numpy",
                 explanation="`Tensor.numpy()` was called, but the NumPy library "
@@ -945,15 +977,10 @@ class TensorVariable(VariableTracker):
 
         def tolist(tensor, sub_proxy):
             def wrap(i, sub_proxy):
-                # Sigh, we forgot to gate this, so this data dependent is on
-                # by default and is load bearing in CI
-                with unittest.mock.patch.object(
-                    tx.fake_mode, "allow_scalar_outputs", True
-                ):
-                    return wrap_fx_proxy(
-                        tx,
-                        sub_proxy.item(),
-                    )
+                return wrap_fx_proxy(
+                    tx,
+                    sub_proxy.item(),
+                )
 
             if tensor.dtype not in [
                 torch.int8,
@@ -961,7 +988,7 @@ class TensorVariable(VariableTracker):
                 torch.int32,
                 torch.int64,
             ]:
-                unimplemented_v2(
+                unimplemented(
                     gb_type="Tensor.tolist() with non-integer tensor",
                     context=f"call_method {self} to_list",
                     explanation="Dynamo currently does not support tracing "
@@ -988,7 +1015,7 @@ class TensorVariable(VariableTracker):
         return VariableTracker.build(tx, out)
 
     def method_backward(self, *args, **kwargs):
-        unimplemented_v2(
+        unimplemented(
             gb_type="Unsupported Tensor.backward() call",
             context=f"call_method {self} backward {args} {kwargs}",
             explanation="Dynamo currently does not support tracing `Tensor.backward()`.",
@@ -999,9 +1026,13 @@ class TensorVariable(VariableTracker):
         return DataPtrVariable(self)
 
     def method_item(self, *args, **kwargs):
-        if not config.capture_scalar_outputs:
+        from ..symbolic_convert import InstructionTranslator
+
+        tx = InstructionTranslator.current_tx()
+        # We enable capture_scalar_outputs when full_graph=True by default.
+        if not tx.one_graph and not config.capture_scalar_outputs:
             self._warn_capture_scalar_outputs()
-            unimplemented_v2(
+            unimplemented(
                 gb_type="Unsupported Tensor.item() call with capture_scalar_outputs=False",
                 context=f"call_method {self} item {args} {kwargs}",
                 explanation="Dynamo does not support tracing `Tensor.item()` "
@@ -1067,6 +1098,14 @@ class TensorVariable(VariableTracker):
         tx = InstructionTranslator.current_tx()
         return self.call_method(tx, "size", [ConstantVariable.create(0)], {})
 
+    def method___iter__(self):
+        from ..symbolic_convert import InstructionTranslator
+
+        tx = InstructionTranslator.current_tx()
+        return ListIteratorVariable(
+            self.unpack_var_sequence(tx), mutation_type=ValueMutationNew()
+        )
+
     def method_addcmul_(self, tensor1, tensor2, *, value=None):
         from ..symbolic_convert import InstructionTranslator
 
@@ -1090,7 +1129,7 @@ class TensorVariable(VariableTracker):
             *proxy_args_kwargs([self, key, value], {}),
         )
 
-        if isinstance(value, TensorVariable):
+        if value.is_tensor():
             # [Note: Tensor.__setitem__ and VariableTracker metadata]
             # At this point, we proxied a node representing `self[key] = value` into the graph.
             # When executed, this node will mutate `self`'s tensor metadata, so it's important
@@ -1099,20 +1138,26 @@ class TensorVariable(VariableTracker):
             #   value.requires_grad is True => self.has_grad_fn becomes True
 
             # Not sure if __setitem__ can ever save activations, disabling just in case
-            with torch._dynamo.utils._disable_saved_tensors_hooks_during_tracing():
+
+            # Ignore fresh unbacked symbols that could arise from the internal indexing (selection),
+            # that happen in code like t[idx] += 1 when idx is unbacked. Namely the selection
+            # during 'setitem'.
+            # When the selection happens if idx is unbacked we allocate a new unbacked symbol for the
+            # storage offset in select_meta, but the output of the operation 'setitem' does not depend
+            # on the selection.
+            with (
+                torch._dynamo.utils._disable_saved_tensors_hooks_during_tracing(),
+                tx.fake_mode.shape_env.ignore_fresh_unbacked_symbols()
+                if tx.fake_mode and tx.fake_mode.shape_env
+                else nullcontext(),
+            ):
                 get_fake_value(proxy.node, tx, allow_non_graph_fake=False)
 
-            example_value = self.proxy.node.meta.get("example_value")
-            from .builder import get_specialized_props, infer_subclass_type
+            vt = value
+            if isinstance(vt, variables.lazy.LazyVariableTracker):
+                vt = variables.lazy.LazyVariableTracker.realize_all(vt)
 
-            if isinstance(value, variables.lazy.LazyVariableTracker):
-                value = variables.lazy.LazyVariableTracker.realize_all(value)
-
-            specialized_props = get_specialized_props(
-                type(value), tx, example_value, infer_subclass_type(example_value)
-            )
-            for k, v in specialized_props.items():
-                setattr(self, k, v)
+            self.synchronize_attributes(tx, type(vt))
 
         if config.use_graph_deduplication or config.track_nodes_for_deduplication:
             tx.output.region_tracker.add_node_mutation(proxy.node, 0)
@@ -1120,7 +1165,7 @@ class TensorVariable(VariableTracker):
         return ConstantVariable.create(None)
 
     def method_resize_(self, *args, **kwargs):
-        unimplemented_v2(
+        unimplemented(
             gb_type="Unsupported Tensor.resize_() call",
             context=f"call_method {self} resize_ {args} {kwargs}",
             explanation="Dynamo currently does not support tracing `Tensor.resize_()`.",
@@ -1128,7 +1173,7 @@ class TensorVariable(VariableTracker):
         )
 
     def method_resize_as_(self, *args, **kwargs):
-        unimplemented_v2(
+        unimplemented(
             gb_type="Unsupported Tensor.resize_as_() call",
             context=f"call_method {self} resize_as_ {args} {kwargs}",
             explanation="Dynamo currently does not support tracing `Tensor.resize_as_()`.",
@@ -1136,7 +1181,7 @@ class TensorVariable(VariableTracker):
         )
 
     def method_sparse_resize_(self, *args, **kwargs):
-        unimplemented_v2(
+        unimplemented(
             gb_type="Unsupported Tensor.sparse_resize_() call",
             context=f"call_method {self} sparse_resize_ {args} {kwargs}",
             explanation="Dynamo currently does not support tracing `Tensor.sparse_resize_()`.",
@@ -1144,7 +1189,7 @@ class TensorVariable(VariableTracker):
         )
 
     def method_sparse_resize_and_clear_(self, *args, **kwargs):
-        unimplemented_v2(
+        unimplemented(
             gb_type="Unsupported Tensor.sparse_resize_and_clear_() call",
             context=f"call_method {self} sparse_resize_and_clear_ {args} {kwargs}",
             explanation="Dynamo currently does not support tracing `Tensor.sparse_resize_and_clear_()`.",
@@ -1159,7 +1204,7 @@ class TensorVariable(VariableTracker):
             # overload and is used by FSDP.
             # graph-breaking on aten::set_source_Tensor_storage_offset for now,
             # unless we find that we need to make it work.
-            unimplemented_v2(
+            unimplemented(
                 gb_type="Unsupported Tensor.set_() call",
                 context=f"call_method {self} set_ {args} {kwargs}",
                 explanation="Dynamo currently does not support tracing `Tensor.set_()` "
@@ -1239,6 +1284,19 @@ class TensorVariable(VariableTracker):
         tx = InstructionTranslator.current_tx()
         # rewrite non-primitive args/kwargs to be included in the on-the-fly prim function
         # and rewrite args to have only proxyable args, then insert call_function
+
+        grad_placements_vt = kwargs.get(
+            "grad_placements", ConstantVariable.create(None)
+        )
+        if isinstance(grad_placements_vt, variables.UserDefinedObjectVariable):
+            # grad_placement is a sequence-like structure, iterate over the value
+            grad_placements_vt = variables.BuiltinVariable(tuple).call_function(
+                tx, [grad_placements_vt], {}
+            )
+
+        if kwargs.get("grad_placements") is not None:
+            kwargs["grad_placements"] = grad_placements_vt
+
         args_as_value = [x.as_python_constant() for x in args]
         kwargs_as_value = {k: v.as_python_constant() for k, v in kwargs.items()}
 
@@ -1291,7 +1349,7 @@ class TensorVariable(VariableTracker):
                 # would have no recourse - their forward traces just fine, but will fail at backwards unless
                 # compiled_autograd is enabled. If compiled_autograd fails (there are a lot of failures today)
                 # then they have nothing they can do except disable compile.
-                unimplemented_v2(
+                unimplemented(
                     gb_type="Compilation of intermediate hooks requires compiled autograd",
                     context=f"var_getattr {self} {name}",
                     explanation="Dynamo must be in compiled_autograd to register hooks.",
@@ -1341,7 +1399,7 @@ class TensorVariable(VariableTracker):
             requires_grad = requires_grad.as_python_constant()
 
         if self.as_proxy().node.meta["example_value"].requires_grad != requires_grad:
-            unimplemented_v2(
+            unimplemented(
                 gb_type="Unsupported Tensor.requires_grad_() call",
                 context=f"call_method {self} requires_grad_",
                 explanation="Dynamo does not support changes to a Tensor's "
@@ -1357,7 +1415,8 @@ class TensorVariable(VariableTracker):
         if (len(args) == 1 and isinstance(args[0], SizeVariable)) or (
             len(args) >= 1
             and all(
-                isinstance(a, ConstantVariable) and a.python_type() == int for a in args
+                a.is_python_constant() and isinstance(a.as_python_constant(), int)
+                for a in args
             )
         ):
             from ..symbolic_convert import InstructionTranslator
@@ -1376,6 +1435,20 @@ class TensorVariable(VariableTracker):
             self.proxy.node._rename(name)
             self._is_name_set = True
 
+    def is_python_hashable(self):
+        # Tensors are hashable if they have an example_value (a fake tensor)
+        # Most VT's should have one.
+        # It'd be nice if at some point we could assert that they all have one
+        return self.as_proxy().node.meta["example_value"] is not None
+
+    def get_python_hash(self):
+        return hash(self.as_proxy().node.meta["example_value"])
+
+    def is_python_equal(self, other):
+        a = self.as_proxy().node.meta["example_value"]
+        b = other.as_proxy().node.meta["example_value"]
+        return a is b
+
 
 class SymNodeVariable(VariableTracker):
     """
@@ -1405,7 +1478,10 @@ class SymNodeVariable(VariableTracker):
             sym_num = int(sym_num) if isinstance(sym_num, sympy.Integer) else sym_num
             return ConstantVariable.create(sym_num)
 
-        return SymNodeVariable(proxy, sym_num, **options)
+        out = SymNodeVariable(proxy, sym_num, **options)
+        if proxy.node.op != "placeholder":
+            tx.output.current_tracer.record_tensor_or_symint_vt(out)
+        return out
 
     def __init__(self, proxy, sym_num, **kwargs) -> None:
         super().__init__(**kwargs)
@@ -1420,6 +1496,9 @@ class SymNodeVariable(VariableTracker):
         else:
             return type(self.sym_num)
 
+    def is_symnode_like(self) -> bool:
+        return True
+
     def as_proxy(self):
         return self.proxy
 
@@ -1461,6 +1540,20 @@ class SymNodeVariable(VariableTracker):
             ),
         )
 
+    def is_python_hashable(self):
+        return True
+
+    def get_python_hash(self):
+        # Essentially convert the SymNode to a constant variable whenever its
+        # searched for a dict key.
+        return hash(self.evaluate_expr())
+
+    def is_python_equal(self, other):
+        if isinstance(other, SymNodeVariable):
+            return self.evaluate_expr() == other.evaluate_expr()
+        # could be constant variable as well
+        return self.evaluate_expr() == other.as_python_constant()
+
 
 class NumpyNdarrayVariable(TensorVariable):
     """
@@ -1533,14 +1626,14 @@ class NumpyNdarrayVariable(TensorVariable):
                 return ConstantVariable.create(int(r))
             return insert_into_graph()
         elif name in ["base", "flags", "dtype"]:
-            unimplemented_v2(
+            unimplemented(
                 gb_type="Unsupported ndarray attribute access",
                 context=f"var_getattr {self} {name}",
                 explanation=f"Dynamo currently does not support tracing `ndarray.{name}`.",
                 hints=[],
             )
-        elif name in ["__version__"]:
-            unimplemented_v2(
+        elif name == "__version__":
+            unimplemented(
                 gb_type="Unsupported ndarray.__version__ access",
                 context=f"var_getattr {self} {name}",
                 explanation=f"Dynamo currently does not support tracing `ndarray.{name}`.",
@@ -1564,7 +1657,7 @@ class NumpyNdarrayVariable(TensorVariable):
         args: "list[VariableTracker]",
         kwargs: "dict[str, VariableTracker]",
     ) -> "VariableTracker":
-        from ..exc import unimplemented_v2
+        from ..exc import unimplemented
         from ..utils import numpy_method_wrapper
 
         args, kwargs = self.patch_args(name, args, kwargs)
@@ -1577,14 +1670,12 @@ class NumpyNdarrayVariable(TensorVariable):
                 dtype_arg = kwargs["dtype"]
             elif len(args) > 0:
                 dtype_arg = args[0]
-            is_object_str = (
-                isinstance(dtype_arg, ConstantVariable) and dtype_arg.value == "O"
-            )
+            is_object_str = dtype_arg is not None and dtype_arg.is_constant_match("O")
             is_object_type = (
                 isinstance(dtype_arg, BuiltinVariable) and dtype_arg.fn is object
             )
             if is_object_str or is_object_type:
-                unimplemented_v2(
+                unimplemented(
                     gb_type="ndarray.astype(object)",
                     context=f"call_method {self} {name} {args} {kwargs}",
                     explanation=(
@@ -1594,11 +1685,11 @@ class NumpyNdarrayVariable(TensorVariable):
                     ),
                     hints=[*graph_break_hints.FUNDAMENTAL],
                 )
-        if name in ["__len__", "size", "tolist"]:
+        if name in ["__len__", "size", "tolist", "__iter__"]:
             # delegate back to TensorVariable
             return super().call_method(tx, name, args, kwargs)
         if name in ("tostring", "tobytes", "__delattr__"):
-            unimplemented_v2(
+            unimplemented(
                 gb_type="Unsupported ndarray method call",
                 context=f"call_method {self} {name} {args} {kwargs}",
                 explanation=f"`ndarray.{name}()` is not modelled in `torch._numpy`.",
@@ -1674,11 +1765,7 @@ class TensorSubclassVariable(UserDefinedClassVariable):
 
         new_func = self.value.__new__
         if new_func is torch.Tensor.__new__:
-            if (
-                len(args) == 1
-                and isinstance(args[0], TensorVariable)
-                and len(kwargs) == 0
-            ):
+            if len(args) == 1 and args[0].is_tensor() and len(kwargs) == 0:
                 data = args[0]
                 # Simulate `torch.Tensor.__new__` as shallow-copying the input
                 # tensor data with a new type. TODO polyfill?
@@ -1686,7 +1773,7 @@ class TensorSubclassVariable(UserDefinedClassVariable):
                     tx, data, self.value, self.source
                 )
             else:
-                unimplemented_v2(
+                unimplemented(
                     gb_type="Calling subclass default constructor with more than tensor argument",
                     context=f"{self.value}(args={args}, kwargs={kwargs})",
                     explanation="Currently not supported",
@@ -1744,8 +1831,13 @@ class UntypedStorageVariable(VariableTracker):
         kwargs: dict[str, VariableTracker],
     ) -> VariableTracker:
         if name == "size":
-            assert not args
-            assert not kwargs
+            if args or kwargs:
+                raise_args_mismatch(
+                    tx,
+                    name,
+                    "0 args and 0 kwargs",
+                    f"{len(args)} args and {len(kwargs)} kwargs",
+                )
             result = self.example_value.size()
             if not has_free_symbols(result):
                 # avoid creating a node in the graph
@@ -1764,7 +1856,8 @@ class UntypedStorageVariable(VariableTracker):
                     ),
                 )
         if name == "resize_" and len(args) == 1:
-            assert not kwargs
+            if kwargs:
+                raise_args_mismatch(tx, name, "0 kwargs", f"{len(kwargs)} kwargs")
             tx.output.create_proxy(
                 "call_function",
                 torch.ops.inductor.resize_storage_bytes_,
